$TITLE ShortestCompletionTime OA4202 2011
$INLINECOM { }
 OPTIONS
   SOLPRINT =     OFF,
   DECIMALS =       1,
   LIMCOL   =      10,
   LIMROW   =      10,
   RESLIM   =      60, {max seconds}
   ITERLIM  =99999999, {max iterations}
   LP       =     CPLEX,
   MIP      =     CPLEX
 ;


$offlisting


$ontext
============================================================================
We define static sets for the nodes and arcs.  We also create some aliases for the nodes set.
============================================================================
$offtext

SET
    n   nodes  /
$INCLUDE  gams_nodesSet.csv
 /
 ;

alias(n,i,j,a,b)   {(i,j) is for describing arcs}

SET
    arcs(i,j)   arcs  /
$ONDELIM
$INCLUDE  gams_arcsSet.csv
$OFFDELIM
 /
 ;

$ontext
============================================================================
We create a list of acronyms that are going to appear as values in a data
table. Typically, GAMS only allows us to store real numbers as values in a
table.  If we want to store strings, as we do in this example, we have to first
declare all those strings as being acronyms.
============================================================================
$offtext

ACRONYMS
$INCLUDE  gams_acronyms.csv
;

$ontext
============================================================================
We now read in the data table for the nodes and arcs.
============================================================================
$offtext

table arcsdata(i,j,*)
$ONDELIM
$INCLUDE  gams_arcsData.csv
$OFFDELIM
;

table nodesdata(i,*)
$ONDELIM
$INCLUDE  gams_nodesData.csv
$OFFDELIM
;


$ontext
============================================================================
Now, we're ready to define the variables and constraints in our formulation.
============================================================================
$offtext

VARIABLE
  Z                 objective function value
;

POSITIVE VARIABLES
  Y(i,j)            flow on arc i j
;

EQUATIONS
  OBJECTIVE
  NETFLOW(n)
;

OBJECTIVE..
 Z =e= SUM(arcs(i,j), arcsdata(i,j,'Cost')*Y(i,j)) 
;

NETFLOW(j)..
 SUM(arcs(i,j), Y(i,j)) - SUM(arcs(j,i), Y(j,i)) =e= 0 - 1$(diag(j,'Start')) + 1$(diag(j,'End'))
;

MODEL ShortestPath
/
  OBJECTIVE
  NETFLOW
/;

$ontext
============================================================================
Now, lets loop and find which pair gives the longest path.
============================================================================
$offtext

SOLVE ShortestPath USING LP MINIMIZING Z;       { find shortest path }

$ontext
============================================================================
Finally, lets print a report a file called shortest_completion_time.out.
============================================================================
$offtext


file out /shortest_completion_time.out/;
put out;

put 'Shortest Completion Time'/;
put '****' / ;

$ontext
============================================================================
GAMS doesn't like to do computation in print statements.  But, all of our
shortest paths are negative, so to print them out as positive, let's declare a
scalar to hold the computational value.
============================================================================
$offtext

scalar positive_value;

put 'Critical Path'/;
LOOP(arcs(i,j)$(Y.l(i,j)>0),
  positive_value = -arcsdata(i,j,'Cost');
  put i.tl, ' Long Name: ', nodesdata(i, 'LongName'):20, ' before ', j.tl, ' Long Name: ', nodesdata(j,'LongName'):20, ' requires time ', positive_value / ;
);

positive_value = -Z.l;
put 'total cost=', positive_value:10:1 / ;

